#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/fb.h>

/* Any frame buffer will do */
static const char *fbpaths[] = {
	"/dev/fb0",	/* Regular name */
	"/dev/fb/0"	/* Devfs name */
};
#define NUMPATHS	(sizeof(fbpaths)/sizeof(*fbpaths))

static char *fberrors[NUMPATHS];

static const char *programname;

static void Usage(void)
{
    fprintf(stderr, "\nUsage: %s console [framebuffer]\n\n", programname);
    exit(1);
}

int main(int argc, char *argv[])
{
    int do_write = 0;
    int fd, i;
    struct fb_con2fbmap map;

    programname = argv[0];
    switch (argc) {
	case 3:
	    do_write = 1;
	    map.framebuffer = atoi(argv[2]);
	case 2:
	    map.console = atoi(argv[1]);
	    break;
	default:
	    Usage();
    }
    for (fd = -1, i = 0; i < NUMPATHS; i++) {
	    if ((fd = open(fbpaths[i], O_RDONLY)) >= 0) break;
	    fberrors[i] = strerror(errno);
    }
    if (fd < 0) {
	    for (i = 0; i < NUMPATHS; i++) {
		    fprintf(stderr, "open %s: %s\n", fbpaths[i], fberrors[i]);
	    }
	    exit(1);
    }

    if (do_write) {
	if (ioctl(fd, FBIOPUT_CON2FBMAP, &map)) {
	    fprintf(stderr, "ioctl FBIOPUT_CON2FBMAP: %s\n", strerror(errno));
	    exit(1);
	}
    } else {
	if (ioctl(fd, FBIOGET_CON2FBMAP, &map)) {
	    fprintf(stderr, "ioctl FBIOGET_CON2FBMAP: %s\n", strerror(errno));
	    exit(1);
	}
	printf("console %d is mapped to framebuffer %d\n", map.console,
	       map.framebuffer);
    }
    close(fd);
    exit(0);
}
